@@ -1,7 +1,7 @@
 /*
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
   This program is free software; you can redistribute it and/or 
   modify it under the terms of the GNU General Public License 
@@ -1430,7 +1430,7 @@ static int _header_name_get_id(const DbmailMessage *self, const char *header, ui
 	gchar *case_header, *safe_header, *frag;
 	Connection_T c; ResultSet_T r; PreparedStatement_T s;
 	Field_T config;
-	volatile gboolean cache_readonly = false;
+	volatile gboolean cache_readonly = true;
 	volatile int t = FALSE;
 
 	// rfc822 headernames are case-insensitive
@@ -1438,8 +1438,8 @@ static int _header_name_get_id(const DbmailMessage *self, const char *header, ui
 
 	config_get_value("header_cache_readonly", "DBMAIL", config);
 	if (strlen(config)) {
-		if (SMATCH(config, "true") || SMATCH(config, "yes")) {
-			cache_readonly = true;
+		if (SMATCH(config, "false") || SMATCH(config, "no")) {
+			cache_readonly = false;
 		}
 	}
 
@@ -1612,7 +1612,7 @@ static gboolean _header_insert(uint64_t physmessage_id, uint64_t headername_id,
 		db_stmt_exec(s);
 		db_commit_transaction(c);
 	CATCH(SQLException)
-		LOG_SQLERROR;
+		LOG_SQLWARNING;
 		db_rollback_transaction(c);
 		t = FALSE;
 	FINALLY
