@@ -2,7 +2,7 @@
  Copyright (C) 1999-2004 IC & S  dbmail@ic-s.nl
  Copyright (c) 2004-2013 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -98,8 +98,6 @@ int do_showhelp(void) {
 	"     -v        verbose details\n"
 	"     -V        show the version\n"
 	"     -h        show this help message\n"
-	"\nSpecific Tasks:\n"
-	"     --upgrade-schema        upgrade sql schema\n"
 
 	);
 
@@ -722,12 +720,12 @@ int do_check_integrity(void)
 	/* end part 5 */
 
 	/* part 6 */
-        Field_T config;
-	gboolean cache_readonly = false;
+	Field_T config;
+	gboolean cache_readonly = true;
 	config_get_value("header_cache_readonly", "DBMAIL", config);
 	if (strlen(config)) {
-		if (SMATCH(config, "true") || SMATCH(config, "yes")) {
-			cache_readonly = true;
+		if (SMATCH(config, "false") || SMATCH(config, "no")) {
+			cache_readonly = false;
 		}
 	}
 
