@@ -2,7 +2,7 @@
  Copyright (c) 2002 Aaron Stone, aaron@serendipity.cx
  Copyright (c) 2004-2010 NFG Net Facilities Group BV support@nfg.nl
  Copyright (c) 2014-2019 Paul J Stevens, The Netherlands, support@nfg.nl
- Copyright (c) 2020-2022 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
+ Copyright (c) 2020-2023 Alan Hicks, Persistent Objects Ltd support@p-o.co.uk
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License 
@@ -273,33 +273,37 @@ static LDAPMessage * authldap_search(const gchar *query)
 	char **_ldap_attrs = NULL;
 	int err = -1; // Start wanting success
 	int c = 0;
+	const char *err_msg = NULL;
 	int c_tries = _ldap_cfg.query_timeout_int;
 	LDAP *_ldap_conn;
 
 	g_return_val_if_fail(query!=NULL, NULL);
 
-	_ldap_conn = ldap_con_get();
-
 	TRACE(TRACE_DEBUG, " [%s]", query);
 
 	while (err != 0 && c++ < c_tries) {
 		// Loop until success or too many retries
+		_ldap_conn = ldap_con_get();
 
 		// timeout must be NULL as any value times out!
-		err = ldap_search_ext_s(_ldap_conn, _ldap_cfg.base_dn, _ldap_cfg.scope_int, 
-				query, _ldap_attrs, _ldap_attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &ldap_res);
+		err = ldap_search_ext_s(_ldap_conn, _ldap_cfg.base_dn, _ldap_cfg.scope_int,
+			query, _ldap_attrs, _ldap_attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &ldap_res);
 
 		switch (err) {
 			case LDAP_SUCCESS:
 				return ldap_res;
 				break;
 			case LDAP_SERVER_DOWN:
-				TRACE(TRACE_WARNING, "LDAP gone away: %s. Trying again(%d/%d).", ldap_err2string(err), c, c_tries);
+				ldap_get_option(_ldap_conn, LDAP_OPT_DIAGNOSTIC_MESSAGE, &err_msg);
+				TRACE(TRACE_WARNING, "LDAP gone away(%d): %s. Trying again(%d/%d). Error message: %s", err, ldap_err2string(err), c, c_tries, err_msg);
+				ldap_memfree(&err_msg);
 				break;
 			default:
 				TRACE(TRACE_ERR, "LDAP error(%d): %s. Trying again (%d/%d).", err, ldap_err2string(err), c, c_tries);
 				break;
 		}
+		ldap_msgfree(ldap_res);
+		ldap_unbind_ext(_ldap_conn, NULL, NULL);
 		sleep(1); // Search failed. Wait before trying again.
 	}
 
@@ -375,7 +379,6 @@ static char *dm_ldap_get_filter(const gchar boolean, const gchar *attribute, GLi
 	g_string_free(t,TRUE);
 	g_string_free(q,FALSE);
 	g_list_foreach(l,(GFunc)g_free,NULL);
-	g_list_free(l);
 
 	return s;
 }
@@ -900,8 +903,8 @@ int auth_check_user_ext(const char *address, GList **userids, GList **fwds, int
  		query = q->str;
  		g_string_free(t,TRUE);
  		g_string_free(q,FALSE);
- 		g_list_foreach(l,(GFunc)g_free,NULL);
- 		g_list_free(l);	
+		g_list_foreach(l,(GFunc)g_free,NULL);
+		g_list_free(l);
  	} else {
  		int i;
  		GString *q = g_string_new("");
